/************************************************************************
 * Main.java
 * This is the controller class for the whiteboard application.  It is 
 * based on the Main.java class of Batik's SVG Viewer implementation, so
 * I'm including the copyright below.
 * 
 * This class initializes resources, creates instances of ViewerFrame,
 * GraphicsManager, SVGWBText, and SVGGraphics2D objects.  Provides
 * accessor methods for all of these. 
 ***********************************************************************/

//package org.apache.batik.apps.svgviewer;

import java.awt.*;
import java.awt.event.*;
import java.net.*;
import java.io.*;
import java.util.*;
import javax.swing.*;

import org.xml.sax.*;

import org.w3c.dom.css.*;
import org.w3c.dom.views.*;
import org.w3c.dom.*;
import org.w3c.dom.*;
import org.w3c.dom.svg.*;
import org.apache.xerces.parsers.*;

import org.apache.batik.css.*;
import org.apache.batik.experiment.*;
import org.apache.batik.util.*;
import org.apache.batik.util.awt.svg.*;
import org.apache.batik.dom.*;
import org.apache.batik.dom.svg.*;

import org.apache.batik.util.gui.resource.*;
//import org.apache.batik.util.gui.resource.ResourceManager;
//import org.apache.batik.apps.svgviewer.ViewerFrame;
//import org.apache.batik.apps.svgviewer.Application;
import org.apache.batik.dom.util.*;

/**
 * This class represents a SVG viewer.
 *
 * @author <a href="mailto:stephane@hillion.org">Stephane Hillion</a>
 * @author <a href="mailto:pierceme@asc.hpc.mil">Marlon Pierce </a>
 */
public class Main extends JFrame implements Application {
    /**
     * The CSS parser class name key.
     */
    public final static String CSS_PARSER_CLASS_NAME_KEY =
        "org.w3c.css.sac.parser";

    /**
     * The XML parser class name key.
     */
    public final static String XML_PARSER_CLASS_NAME_KEY =
        "org.xml.sax.driver";

    /**
     * The gui resources file name
     */
    public final static String RESOURCES1 =
        "org.apache.batik.apps.svgviewer.resources.Application";
	public final static String RESOURCES2=
		"org.apache.batik.apps.svgviewer.resources.GUI";

    /**
     * The resource bundle
     */
    protected static ResourceBundle bundle1,bundle2;
    protected static ResourceManager resources1,resources2;
    static {
        bundle1 = ResourceBundle.getBundle(RESOURCES1, Locale.getDefault());
        resources1=new ResourceManager(bundle1);
        bundle2 = ResourceBundle.getBundle(RESOURCES2, Locale.getDefault());
        resources2=new ResourceManager(bundle2);
    }
    
    /**
     * The XML parser class name.
     */
    protected static String xmlParserClassName =
        bundle1.getString(XML_PARSER_CLASS_NAME_KEY);

    /** 
     * The following are data needed and accessible by all parts of
     * the Main class.  The names are mostly self-explanatory.
     */
    protected SVGOMDocument document=null;
    protected DOMImplementation domImpl;
    protected SVGGraphics2D svgGenerator;
    public AppletFrame vfApplet;
    protected int mainFrames;  //Number of mainFrames
    protected ViewerFrame mainFrame;
    protected GraphicsManager graphman;
    protected SVGWBText text;
    protected Font font=new Font("SansSerif",Font.PLAIN,18);
    
    /**
     * An empty constructor for testing.
     */
  //public Main(){
	//Does nothing.
  //}
    
    /**
     * Class constructor.
     * Creates a viewer application frame.
     * @param args The command-line arguments.
     */
  //public Main(String[] args) {
  public Main() {
	Experiments.registerExperimentBridges();
        CSSDocumentHandler.setParserClassName
            (bundle1.getString(CSS_PARSER_CLASS_NAME_KEY));

	//Initialize the various classes needed to create and display 
	//the image. 
	document=new SVGOMDocument();
	svgGenerator=new SVGGraphics2D(document);

	mainFrame=new ViewerFrame(this);	
	this.setTitle("White Board");
	this.setSize(resources2.getInteger("Frame.width"),
		     resources2.getInteger("Frame.height"));
	graphman=new GraphicsManager(this);
	text=new SVGWBText(this);

	mainFrame.init();

	this.setVisible(true);
    }
    
    /**
     * The program entry point.
     * @param args The command-line arguments.
     */
  /*
    public static void main(String[] args) {
	
	if(args.length!=0) {
	    System.out.println("Usage: java Main ");
	    System.exit(1);
	}
        Main mn=new Main(args);
    }
    */
    
    ////////////////////////////////////////////////////////////
    // Application /////////////////////////////////////////////
    // These methods must be defined as part of the Application 
    // interface.
    ////////////////////////////////////////////////////////////

    /**
     * Creates and shows a new viewer frame.
     */
    public void createAndShowViewerFrame() {
        mainFrames++;
        ViewerFrame vf = new ViewerFrame(this);
	//        vf.show();
	vf.setVisible(true);
        mainFrame = vf;
    }

    /**
     * Creates a close action.
     */
    public Action createCloseAction(ViewerFrame vf) {
        return new CloseAction(vf);
    }

    /**
     * Creates an exit action.
     */
    public Action createExitAction() {
        return new ExitAction();
    }

    /**
     * Returns the XML parser class name.
     */
    public String getXMLParserClassName() {
        return xmlParserClassName;
    }

    /**
     * Called when the frame will be closed.
     * @return true if the frame must be closed.
     */
    public boolean closeFrame(ViewerFrame f) {
        System.exit(0);
        return true;
    }

    /**
     * Opens the given link.
     */
    public void openLink(ViewerFrame f, SVGAElement elt) {
        String show = elt.getXlinkShow();
        String href = elt.getHref().getBaseVal();

        String docURL = ((SVGDocument)elt.getOwnerDocument()).getURL();
        URL uri = null;
        try {
            uri = new URL(new URL(docURL), href);
        } catch (MalformedURLException e) {
            System.out.println(e.getMessage());
        }

        if (show.length() > 0 && show.charAt(0) == 'n') {
            createAndShowViewerFrame();
            mainFrame.loadDocument(uri.toString());
        } else {
            f.loadDocument(uri.toString());
        }
    }
    ////////////////////////////////////////////////////////////////
    // Accessor methods ////////////////////////////////////////////
    // These methods provide access to various local variables.
    ////////////////////////////////////////////////////////////////

    /** 
     * Returns the instance of the SVGOMDocument.  Note that I'm using
     * SVGOMDocument to be compatible with ViewerCanvas.  As I learn more
     * about the API, I will probably modify this.
     */
    public SVGOMDocument getSVGOMDocument() {
	return document;
    }

    /**
     * Sets the value of the SVGOMDocument.  Main can aquire this from
     * either a file read in from disk or from an internally created new
     * instance.
     */
    public void setSVGOMDocument(SVGOMDocument doc) {
	if(document!=null) document=null;
	document=doc;
    }

    /**
     * Gets value of the DOM implementation
     */
    public DOMImplementation getDOMImpl(){ return domImpl; }

    /**
     * Accessor method for the SVG graphics object.
     */
    public SVGGraphics2D getSVGGenerator() { return svgGenerator; }

    /**
     * Get a reference to the SVGViewer object.
     */
    public ViewerFrame getViewerFrame() { return mainFrame; }

    /**
     * Access graphics manager
     */
    public GraphicsManager getGraphicsManager() { return graphman; }

    /**
     * Access the viewer frame.
     */
    public ViewerFrame getMainFrame() { return mainFrame; }
    
    /**
     * Access the AppletFrame
     */
    public AppletFrame getContainerFrame() { return vfApplet; }
    
    /**
     * Test local copy of the main application.
     */
    public void test() { System.out.println("Main found"); }

    /**
     * Accessor method for the SVGWBText object
     */
    public SVGWBText getSVGWBText() { return text; }
    
    /**
     * Accessor for the Font object.
     */
    public Font getFont() { return font; }

    // Actions /////////////////////////////////////////////////
    
    /**
     * To close a frame.
     */
    public class CloseAction extends AbstractAction {
        protected ViewerFrame viewerFrame;
        public CloseAction(ViewerFrame vf) { viewerFrame = vf; }
        public void actionPerformed(ActionEvent e) {
            if (closeFrame(viewerFrame)) {
                //viewerFrame.setVisible(false);
                vfApplet.setVisible(false);
            }
        }
    }

    /**
     * To exit the application
     */
    public class ExitAction extends AbstractAction {
        public ExitAction() {}
        public void actionPerformed(ActionEvent e) {
            System.exit(0);
        }
    }
}

