/**
 * Program Name: createScript.java
 * Comments:
 * This is a parent class for all of the createJob scripts for the different
 * platforms.  It receives the ContextManager object from the calling JSP page
 * (or whatever).  This object is protected, and can be accesses by the 
 * children.  
 * 
 * We want to write the script to the user's "Descriptors" directory, which 
 * is located under the WEB-INF directory.  It is thus not accessible by
 * the web server and thus by the JSP pages.  So we have to use a bean.
 *
 * The String scriptString is used to store the script as it is constructed.
 * Part of this occurs here (the createCommands() method) and part must 
 * occur in the appropriate child (the createHeader() method).  So this 
 * string is protected and inherited by the child classes.
 *
 * I am assuming the following naming convention for the ContextData:
 * Directory: the user's decriptors directory (xml).
 * IOStyle: type of I/O.  Can by C-style, standard I/O, or internal(xml).
 * WorkDir: name of the workspace directory in which the program will
 *          execute (xml).
 * Name: name of problem (user).
 * NumInParams: number of input parameters (xml).
 * NumOutFiles: number of output files (xml).
 * inputParam: values of the input parameters, include input filenames.
 *             Possibly multivalues. (user or xml).
 * outputFiles: names of the output files.  Possible multivalued (user or xml).
 * execpath: full path of the executable (xml).
 * 
 * The data source can either be from ApplDesc.xml (xml) or from the 
 * user (user).
 */

package extensions.generic;

import WebFlow.*;
import WebFlow.ContextManager.*;
import WebFlow.RemoteFile.*;
import WebFlow.RemoteFile.RemoteFilePackage.*;
import java.io.*;
import java.util.*;

public abstract class createScript {
    protected ContextManager cm;
    protected RemoteFile rf;
    protected String descDir;
    protected PrintWriter pw;
    protected String scriptString=new String();
    String EventGenString; //="/a/birch.csit.fsu.edu/shared/home/cyoun/GATEWAY/apache1.3.12/WWW/GOW/WEB-INF/Descriptors/EventGenerator";
    String MonServUrl; //="http://birch.csit.fsu.edu:8001/GOW/servlet/WebFlow.Servlets.MonitorJobServ";
    String userName;
    String javaCommand; //="/home/scri97/users/cyoun/birch/globus/akarsu/jdk1.2/jre/bin/java -cp ";
    //String DataFilePath="/a/birch.csit.fsu.edu/shared/home/cyoun/GATEWAY/apache1.3.12/WWW/GOW/Viz/VizData/";
    //String OutFilePath="/tmp/";



    public createScript() { }
   
    public createScript(ContextManager cm, String name) {
	this.cm=cm;
	userName=name;
	descDir=cm.getCurrentSessionProperty("Directory");
	try {
	    pw=new PrintWriter(new FileWriter(descDir+"/job.script"));
	}
	catch(Exception ex) {
	    System.err.println("Could not open file for printing");
	    ex.printStackTrace();
	}
    }

    public createScript(ContextManager cm, RemoteFile rf, String name) {
	this.cm=cm;
	this.rf = rf;
	userName=name;
	descDir=cm.getCurrentSessionProperty("Directory");
	try {
	    //pw=new PrintWriter(new FileWriter(descDir+"/job.script"));
	    rf.open(descDir+"/job.script","rw");
	}
	catch(Exception ex) {
	    System.err.println("Could not open file for printing");
	    ex.printStackTrace();
	}
    }

    public createScript(ContextManager cm, String name, String fname) {
	this.cm=cm;
	userName=name;
	descDir=cm.getCurrentSessionProperty("Directory");
	try {
	    pw=new PrintWriter(new FileWriter(descDir+"/"+fname));
	}
	catch(Exception ex) {
	    System.err.println("Could not open file for printing");
	    ex.printStackTrace();
	}
    }

    public createScript(ContextManager cm, RemoteFile rf, String name, String fname) {
	this.cm=cm;
	this.rf = rf;
	userName=name;
	descDir=cm.getCurrentSessionProperty("Directory");
	try {
	    //pw=new PrintWriter(new FileWriter(descDir+"/"+fname));
	    rf.open(descDir+"/"+fname,"rw");
	}
	catch(Exception ex) {
	    System.err.println("Could not open file for printing");
	    ex.printStackTrace();
	}
    }

    /* set properties for creating script */
    public void setScriptProperties(String propspath)
    {
	//Properties reading
	Properties props = new Properties();
	try {
	    props.load(new FileInputStream(propspath));
	    EventGenString = props.getProperty("EventGenString");
	    MonServUrl = props.getProperty("MonServUrl");
	    javaCommand = props.getProperty("javaCommand");
	}
	catch(Exception ex) {
	    System.out.println("Error reading the property file(createScript).");
	}
    }

    /** 
     * The child should construct the script as a single string and
     * use the following method to print it out.
     */
    public void writeScript() {
	pw.print(scriptString);
	pw.flush();
	pw.close();
    }

    public void writeScriptRf() {
	byte buffer[] = scriptString.getBytes();
	try {
	    rf.putBlock(buffer);
	    rf.close();
	}catch (FileException ex) { 
	    rf.close(); 
	}
    }


    /**
     * The following method should be overriden by each child.  This
     * creates a tag that associates each flag with its name for a 
     * given queuing system.
     */
    public abstract void createFlagTable();

    /**
     * The following method is used to construct the header information
     * with the queue specific directives.  For example, a PBS script
     * begins with
     *   #PBS -q default
     *   #PBS -l walltime=10:00:00
     * and so on.  Must be overriden by each child.
     */
    public abstract void createHeader();

    /**
     * The following method should be used to create the command portion
     * of the job script.  This part is code specific, not queue specific,
     * so we can do this here.
     * 
     * Note this requires that the ContextData has the following properties
     * set: IOStylem, WorkDir, Name.  This is FRAGILE since changes in 
     * the JSP pages that collect info and store them in the ContextData can
     * potentially break this method.  Currently we have to really on 
     * a name convention.  We have no way of enforcing this convention.  
     * Probably should move to XML/DTD for this reason.
     */
    public void createCommands() {
	String IOStyle=cm.getCurrentSessionProperty("IOStyle");
	String workDir=cm.getCurrentSessionProperty("WorkDir");
	String jobName=cm.getCurrentProblemProperty("Name");
	String[] inputParams = null;
	String[] inputFiles = null;
	String[] outputFiles = null;
	String[] outputParams = null;

	String numinparams = cm.getCurrentSessionProperty("NumberOfInParams");
	String numinfiles = cm.getCurrentSessionProperty("NumberOfInFiles");
	String numoutfiles = cm.getCurrentSessionProperty("NumberOfOutFiles");
	String numoutparams = cm.getCurrentSessionProperty("NumberOfOutParams");

	int inparamArg = Integer.parseInt(numinparams);
	int infileArg = Integer.parseInt(numinfiles);
	int outfileArg = Integer.parseInt(numoutfiles);
	int outparamArg = Integer.parseInt(numoutparams);

	String inSeparator="",outSeparator="";
	
	if (inparamArg != 0) inputParams=parseMultiVal("inputParam",inparamArg);
	if (infileArg != 0) inputFiles=parseMultiVal("inputFiles",infileArg);
	if (outfileArg != 0) outputFiles=parseMultiVal("outputFiles",outfileArg);
	if (outparamArg != 0) outputParams=parseMultiVal("outputParams",outparamArg);

	//Set up the working directory
	workDir+="/"+jobName;
	scriptString+="if (! -d "+workDir+") then\n";
	scriptString+="  mkdir -p "+workDir+"\n";
	scriptString+="endif\n";
	scriptString+="cd "+workDir+"\n";
	
        //Move inputfiles to proper location
        for(int k=0;k<infileArg;k++)
            scriptString+="cp "+inputFiles[k]+" "+ workDir+"\n";    

	//Put in event notification
	scriptString+="##DO NOT DELETE THE FOLLOWING LINE\n";
	scriptString+=javaCommand+" "+EventGenString+"/EventGenerator "
	    +MonServUrl+" "+userName+" "
	    +cm.getCurrentProblemProperty("Name")+" started\n";

	if(IOStyle.equals("CArgument")){
	    inSeparator=" ";
	    outSeparator=" ";
	}
	else if(IOStyle.equals("StandardIO")){
	    inSeparator=" < ";
	    outSeparator=" > ";
	}
	else if(IOStyle.equals("Internal")){
	}
	else {
	    System.err.println("Invalid IOStyle detected");
	}

	scriptString+=cm.getCurrentSessionProperty("ExecPath");
 
	for(int i=0;i<inparamArg;i++)
	    scriptString=scriptString+inSeparator+inputParams[i];
 
	for(int j=0;j<outparamArg;j++) 
	    scriptString=scriptString+outSeparator+outputParams[j];
	
	scriptString+="\n";

	scriptString+="##DO NOT DELETE THE FOLLOWING LINE\n";
	scriptString+=javaCommand+" "+EventGenString+"/EventGenerator "
	    +MonServUrl+" "+userName+" "
	    +cm.getCurrentProblemProperty("Name")+" finished\n";
    }   
    
    /**
     * ContextData is basically a hashtable, but often we need to store
     * multiple values for a single name.  For example, if we have multiple
     * input files, we would like to store them under a single name and 
     * restore them in an array.  
     * 
     * We will adopt the convention (following the way child contexts are
     * handled) by separating multiple values with a "+" sign.  For example,
     * we may have
     *             inputParam infile1+infile2+infile3
     */
    private String[] parseMultiVal(String cdParamName,int dim){
	String toEat=cm.getCurrentSessionProperty(cdParamName);
	String[] multiVal=new String[dim];
	StringTokenizer st=new StringTokenizer(toEat,"+");
	for(int i=0;i<dim;i++) {
	    multiVal[i]=st.nextToken();
	}
	return multiVal;
    }

    /* This is gnuplot interface
     *
     */

    public void createGnuplotCommands() {
        String datafile = cm.getCurrentSessionProperty("DataFileOne");
        //datafile = DataFilePath+datafile;
        String outfile = cm.getCurrentSessionProperty("VizOutFile");
        outfile = outfile+".ps";

        //String jobName=cm.getCurrentProblemProperty("Name");

        scriptString+="set output \""+outfile+"\""+"\n";
        scriptString+="plot \""+datafile+"\""+" using 1:2 title '"+cm.getCurrentSessionProperty("psptitle")+"' with "+cm.getCurrentSessionProperty("psds")+" "+cm.getCurrentSessionProperty("pscolor")+" "+cm.getCurrentSessionProperty("pspt")+"\n";
        scriptString+="\n";
 
    } 

    /* This is for matlab interface.
     * 
     */
    public void createMatlabCommands() {

	String comment = 
	    "% Matlab script file for plotting data in your data file \n"+
	    "% This script is given the basic function for you. \n"+
	    "% if you need more things or change it, You can modify it. \n"+
	    "\n";

        scriptString+=comment;

        String datafile = cm.getCurrentSessionProperty("DataFileOne");
	int index = datafile.lastIndexOf("/");
	String datafilename = datafile.substring(index+1);
	index = datafilename.indexOf(".");
	if(index != -1) datafilename = datafilename.substring(0,index);

        String outfile = cm.getCurrentSessionProperty("VizOutFile");
        outfile = outfile+".ps";

	String title = cm.getCurrentSessionProperty("pstitle");
	String ptitle = cm.getCurrentSessionProperty("psptitle");
	String xlabel = cm.getCurrentSessionProperty("xlabel");
	String ylabel = cm.getCurrentSessionProperty("ylabel");
	String markersize = cm.getCurrentSessionProperty("psms");
	String markerecolor = cm.getCurrentSessionProperty("psmecolor");
	String markerfcolor = cm.getCurrentSessionProperty("psmfcolor");

	String linewidth = cm.getCurrentSessionProperty("pslw");
	String linetype = cm.getCurrentSessionProperty("psls");
	String markertype = cm.getCurrentSessionProperty("psmt"); 
	String colortype = cm.getCurrentSessionProperty("pscolor");
	String plotoption = "";
	
	if(!linetype.equals("none")) {
	    plotoption = linetype;
	}

	if(!markertype.equals("none")) {
		plotoption += colortype+markertype;
	} else {
	    plotoption += colortype;
	}

        scriptString+="load "+datafile+";\n";
	scriptString+="xaxis = "+datafilename+"(:,1);\n";
	scriptString+="yaxis = "+datafilename+"(:,2);\n";
	scriptString+="plot(xaxis,yaxis,'"+plotoption+"','LineWidth',"+linewidth+",'MarkerEdgeColor','"+markerecolor+"','MarkerFaceColor','"+markerfcolor+"','MarkerSize',"+markersize+");\n";
	scriptString+="xlabel('"+xlabel+"');\n";
	scriptString+="ylabel('"+ylabel+"');\n";
	scriptString+="title('"+title+"');\n";
	scriptString+="legend('"+ptitle+"');\n";			
        scriptString+="print -dpsc "+outfile+"\n";
	scriptString+="exit\n";	
        scriptString+="\n";
 
    } 

}

