/**
 * This is a convenient extension of the regular JPanel class that 
 * allows us to draw a rectangle.  To do this, we implement an inner
 * class that extends MouseInputAdapter and override paintComponent. 
 */ 

import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;

public class GraphPanel extends JPanel {

    public MyListener myListener;
    public Rectangle currSelectRect=null,selectToDraw=null,
	previousSelectDrawn=new Rectangle(),savedRect=null;
    public ViewerFrame vf;

    public GraphPanel(ViewerFrame vf) {
	this.vf=vf;
	myListener=new MyListener();
	this.addMouseListener(myListener);
	this.addMouseMotionListener(myListener);
    }

    public void paintComponent(Graphics g) {
	super.paintComponent(g);
	if (currSelectRect != null) {
	    //Draw a black rectangle on top of the image.
	    g.setColor(Color.black);
            g.drawRect(selectToDraw.x, selectToDraw.y, 
                       selectToDraw.width - 1, selectToDraw.height - 1);
        }
    }

    public Rectangle getGraphRectangle() { return savedRect; }

    /**
     * This class implements some methods for handling mouse events.
     * I use this to draw the rectangle.  I tie this to the 
     * right mouse button.  Note this shares currSelectRect with the
     * ViewerFrame object.
     */ 
    public class MyListener extends MouseInputAdapter {

	/**
	 *mousePressed gets the x,y coordinate for the origination of
	 *a mouseMoved event for click and drag operations
	 */
        public void mousePressed(MouseEvent e) {
	    System.out.println("Mouse pressed");
	    if((e.getModifiers() & InputEvent.BUTTON3_MASK)!=0){
		int x = e.getX();
		int y = e.getY();
		currSelectRect = new Rectangle(x, y, 0, 0);
		updateDrawableRect(getWidth(), getHeight());
		//repaint(); REMOVED FOR OUR MODEL - don't really need repaint
	    }
	}

	/**
	 *mouseDragged udates the size of the area being dragged
	 */
        public void mouseDragged(MouseEvent e) {
	    System.out.println("Mouse dragged");
	    if((e.getModifiers() & InputEvent.BUTTON3_MASK)!=0){
		updateSize(e);	   
		//drag_on = true;
	    }
	}
	
	/**
	 *mouseReleased updates the final area of the drag and paints 
	 *the banded area using paintBandedArea
	 */
        public void mouseReleased(MouseEvent e) {
	    System.out.println("Mouse released");
	    if((e.getModifiers() & InputEvent.BUTTON3_MASK)!=0){

		updateSize(e);		
		savedRect=currSelectRect;

		SVGChemistry svgchem=vf.getMain().getSVGChem();
		svgchem.drawShape(vf.getMain().getSVGGenerator(),1,1);
		
		vf.getLayerCont().moveToBack(GraphPanel.this);

		currSelectRect = null;

		repaint();
	    }
        }

        /**
         * Private method of MyListener:
	 * Update the size of the current rectangle
         * and call repaint.  Because currSelectRect
         * always has the same origin, translate it
         * if the width or height is negative.
         * 
         * For efficiency,
         * specify the painting region using arguments
         * to the repaint() call.
         * 
         */
        private void updateSize(MouseEvent e) {
            int x = e.getX();
            int y = e.getY();
            currSelectRect.setSize(x - currSelectRect.x,
				   y - currSelectRect.y);
            updateDrawableRect(getWidth(), getHeight());
            Rectangle totalRepaint = selectToDraw.union(previousSelectDrawn);
            repaint(totalRepaint.x, totalRepaint.y,
                    totalRepaint.width, totalRepaint.height);
        }
    }

    /**
     * Update the current rectangle to be correct for displaying.
     * Make resulting changes to selectToDraw. The width and height
     * of hte panel are taken into account in cases where the window
     * the draw panel is contained within is resized.
     * @param compWidth The width of the panel.  
     * @param compHeight The height of the panel.
     */
    private void updateDrawableRect(int compWidth, int compHeight) {
        int x = currSelectRect.x;
        int y = currSelectRect.y;
        int width = currSelectRect.width;
        int height = currSelectRect.height;
	
        //Make the width and height positive, if necessary.
        if (width < 0) {
            width = 0 - width;
            x = x - width + 1; 
            if (x < 0) {
                width += x; 
                x = 0;
            }
        }
        if (height < 0) {
            height = 0 - height;
            y = y - height + 1; 
            if (y < 0) {
                height += y; 
                y = 0;
            }
        }
	
        //The rectangle shouldn't extend past the drawing area.
        if ((x + width) > compWidth) {
            width = compWidth - x;
        }
        if ((y + height) > compHeight) {
            height = compHeight - y;
        }
	
        //Update selectToDraw after saving old value.
        if (selectToDraw != null) {
            previousSelectDrawn.setBounds(selectToDraw.x, 
					  selectToDraw.y, 
					  selectToDraw.width, 
					  selectToDraw.height);

            selectToDraw.setBounds(x, y, width, height);
        } 
	else {
            selectToDraw = new Rectangle(x, y, width, height);
        }
    }
}
