/*****************************************************************************
 * Copyright (C) The Apache Software Foundation. All rights reserved.        *
 * ------------------------------------------------------------------------- *
 * This software is published under the terms of the Apache Software License *
 * version 1.1, a copy of which has been included with this distribution in  *
 * the LICENSE file.                                                         *
 *****************************************************************************/
 
 /**
  * Program: ViewerFrame.java
  * @author Marlon Pierce
  * Comments:  Various modifications to the original ViewerFrame class from
  * Batik.  
  *
  * This class manages the display.
  *
  */

//package org.apache.batik.apps.svgviewer;

import java.awt.*;
import java.awt.geom.*;
import java.awt.event.*;
import java.awt.image.*;
import java.io.*;
import java.net.*;
import java.util.*;
import java.util.zip.*;
import javax.swing.*;
import javax.swing.event.MenuEvent;
import javax.swing.event.MenuListener;
import java.awt.event.*;

import javax.swing.border.*;
import javax.swing.text.*;

import org.apache.batik.bridge.*;

import org.apache.batik.dom.svg.*;
import org.apache.batik.gvt.event.*;

import org.apache.batik.refimpl.bridge.*;
import org.apache.batik.refimpl.transcoder.*;
import org.apache.batik.refimpl.util.*;  //Contains JSVGCanvas
import org.apache.batik.refimpl.gvt.event.*;

import org.apache.batik.util.*;
import org.apache.batik.util.gui.*;

import org.apache.batik.util.gui.resource.*;

import org.xml.sax.*;
import org.w3c.dom.DocumentFragment;

import org.w3c.dom.css.ViewCSS;
import org.w3c.dom.svg.SVGSVGElement;
import org.w3c.dom.svg.SVGAElement;
import org.apache.batik.util.awt.svg.SVGGraphics2D;

//import org.apache.batik.apps.svgviewer.Application;

public class ViewerFrame
    extends    JApplet
    implements org.apache.batik.util.gui.resource.ActionMap,
               UserAgent,
               LanguageChangeHandler,
               UserStyleDialog.ChangeHandler,
               JSVGCanvas.ZoomHandler,
	       ActionListener,
               DocumentListener {

    // The actions names.
    public final static String OPEN_ACTION        = "OpenAction";
    public final static String OPEN_PAGE_ACTION   = "OpenPageAction";
    public final static String NEW_WINDOW_ACTION  = "NewWindowAction";
    public final static String EXPORT_PNG_ACTION  = "ExportPNGAction";
    public final static String EXPORT_JPG_ACTION  = "ExportJPGAction";
    public final static String RELOAD_ACTION      = "ReloadAction";
    public final static String BACK_ACTION        = "BackAction";
    public final static String FORWARD_ACTION     = "ForwardAction";
    public final static String CLOSE_ACTION       = "CloseAction";
    public final static String EXIT_ACTION        = "ExitAction";
    public final static String SOURCE_ACTION      = "SourceAction";
    public final static String DESCRIPTION_ACTION = "DescriptionAction";
    public final static String TREE_ACTION        = "TreeAction";
    public final static String THUMBNAIL_ACTION   = "ThumbnailAction";
    public final static String STOP_ACTION        = "StopAction";
    public final static String FIXED_SIZE_ACTION  = "FixedSizeAction";
    public final static String PROG_PAINT_ACTION  = "ProgressivePaintAction";
    public final static String LANGUAGE_ACTION    = "LanguageAction";
    public final static String USER_STYLE_ACTION  = "UserStyleAction";
    public final static String MONITOR_ACTION     = "MonitorAction";
    public final static String ABOUT_ACTION       = "AboutAction";

    /**
     * The following data are accessible by all of the methods of
     * the ViewerFrame class.  The names are self explanatory.  These
     * variables are typically initialized in the constructor or in
     * init().
     */
    protected final static Cursor DEFAULT_CURSOR =
        new Cursor(Cursor.DEFAULT_CURSOR);

    protected final static Cursor WAIT_CURSOR =
        new Cursor(Cursor.WAIT_CURSOR);

    public final static String RESOURCES =
	//        "org.apache.batik.apps.svgviewer.resources.GUI";
        "resources.GUI";

    protected static JFrame memoryMonitor;
    protected static JFrame aboutFrame;
    protected static ResourceBundle bundle;
    protected static ResourceManager resources;
    static {
        bundle    = ResourceBundle.getBundle(RESOURCES, Locale.getDefault());
        resources = new ResourceManager(bundle);
    }
    
    protected static char [] buffer = new char[4096];
    protected Application application;
    protected Main main;
    protected LocationBar locationBar;
    protected JButton textButton;
    protected String uri;
    protected String currentPath = ".";
    protected String currentExportPath = ".";
    protected URIChooser uriChooser;
    protected JPanel panel;
    protected StatusBar statusBar;
    protected JSVGCanvas graphcanvas,textcanvas,thecanvas;
    protected Thread thread;
    protected DOMViewer domViewer = new DOMViewer();
    protected LanguageDialog languageDialog;
    protected UserStyleDialog userStyleDialog;
    protected JFrame thumbnailFrame;
    
    protected ReloadAction reloadAction = new ReloadAction(this);
    protected BackAction backAction = new BackAction(this);
    protected ForwardAction forwardAction = new ForwardAction(this);
    protected StopAction stopAction = new StopAction(this);
    protected IntegralAction intAction=new IntegralAction();
    protected LoadDataAction loadAction=new LoadDataAction();

    protected String description = "";
    protected boolean fixedSize;
    protected boolean progressivePaintEnabled;
    
    private ConcreteEventDispatcher eventDispatcher =
        new ConcreteEventDispatcher();

    protected String userLanguages = "en";
    
    protected String userStyleSheetURI;
    protected java.util.List loadedDocuments = new java.util.ArrayList();
    protected int loadedDocument = -1;
    protected int loadedDocumentsCount;
    private SVGDocumentFactory df;
    public SVGOMDocument textdoc=new SVGOMDocument();
    public SVGOMDocument graphdoc=new SVGOMDocument();
    public SVGOMDocument thedoc=new SVGOMDocument();
    protected DefaultSVGContext dc;

    public JMenuBar jmb;
    public JToolBar tb,mb,drawb;
    public JPanel container,northpane,textpane;
    public JTextField textfield;
    public SVGWBText wbtext;
  public SVGWBDraw wbdraw;
  public Container mainContain;
  public JLayeredPane layercont;
  public DrawPanel drawpanel;
  public SVGGraphics2D sg2d;
    
    //    protected Font font;
    int increment;
    
    /**
     * Creates a new ViewerFrame object.
     * @param main The Main object calling ViewerFrame.
     */
    public ViewerFrame(Main main) {
	//super(new BorderLayout());
        this.main=main;
        sg2d = main.getSVGGenerator();


	//The following assocates action names with action objects
	//in a HashMap.
        listeners.put(OPEN_ACTION,        new OpenAction());
        listeners.put(OPEN_PAGE_ACTION,   new OpenPageAction());
        listeners.put(NEW_WINDOW_ACTION,  new NewWindowAction(this));
	listeners.put(EXPORT_PNG_ACTION,  new ExportPNGAction());
        listeners.put(EXPORT_JPG_ACTION,  new ExportJPGAction(this));
        listeners.put(RELOAD_ACTION,      reloadAction);
        listeners.put(BACK_ACTION,        backAction);
        listeners.put(FORWARD_ACTION,     forwardAction);
        listeners.put(CLOSE_ACTION,       main.createCloseAction(this));
        listeners.put(EXIT_ACTION,        main.createExitAction());
        listeners.put(SOURCE_ACTION,      new SourceAction(this));
        listeners.put(DESCRIPTION_ACTION, new DescriptionAction(this));
        listeners.put(TREE_ACTION,        new TreeAction(this));
        listeners.put(THUMBNAIL_ACTION,   new ThumbnailAction(this));
        listeners.put(STOP_ACTION,        stopAction);
        listeners.put(PROG_PAINT_ACTION,  new ProgressivePaintAction());
        listeners.put(FIXED_SIZE_ACTION,  new FixedSizeAction(this));
        listeners.put(LANGUAGE_ACTION,    new LanguageAction(this));
        listeners.put(USER_STYLE_ACTION,  new UserStyleAction(this));
        listeners.put(MONITOR_ACTION,     new MonitorAction(this));
        listeners.put(ABOUT_ACTION,       new AboutAction());

	listeners.put("PenOnAction", new PenOnAction());        
	listeners.put("PenOffAction", new PenOffAction());
	listeners.put("TextAction", new TextAction());        

	listeners.put("IntegralAction", intAction);
	listeners.put("SqrtAction",new SqrtAction());
	listeners.put("PartAction",new PartAction());
	listeners.put("DelAction",new DelAction());
	listeners.put("LoadDataAction",loadAction);
	listeners.put("SaveSVGAction",new SaveSVGAction());
    }
    
    /**
     * Put all of the initialization code here.
     */
    public void init(){       
	//Initialize the wbtext object.
	wbtext=main.getSVGWBText();
	wbdraw=main.getSVGWBDraw();

	//Initialize the document factory.
        df = new SVGDocumentFactory(main.getXMLParserClassName());

	//	SVGGraphics2D svgbase=new SVGGraphics2D(new SVGOMDocument());
	
	thedoc=initializeDoc(thedoc);

	//Create the container pane that holds all of these components.
	container=new JPanel(new BorderLayout());
	container.setBounds(0,0,resources.getInteger("Frame.width"),
			    resources.getInteger("Frame.height")-30);
	northpane=new JPanel(new BorderLayout());
	 	
	//Give reference of container JFrame to local instance.
	uriChooser = new URIChooser(main, new URIChooserOKAction());
        uriChooser.setFileFilter(new SVGFileFilter());
	
        //Create the SVG canvas and other containers.  Note this MUST
	//come first in order for the action objects above to work.
	thecanvas=new JSVGCanvas(this);
	thecanvas.setBounds(0,0,resources.getInteger("Frame.width"),
  			      resources.getInteger("Frame.height")-60);
  	
//          graphcanvas = new JSVGCanvas(this);
//          textcanvas = new JSVGCanvas(this);
//  	graphcanvas.setBounds(0,0,resources.getInteger("Frame.width"),
//  			      resources.getInteger("Frame.height")-60);
//  	textcanvas.setBounds(0,0,resources.getInteger("Frame.width"),
//  			      resources.getInteger("Frame.height")-60);
//  	textcanvas.setOpaque(false);
//  	graphcanvas.setOpaque(false);

        //Create the menu
	MenuFactory mf = new MenuFactory(bundle, this);
	jmb=mf.createJMenuBar("MenuBar");

        //Create the toolbar
	ToolBarFactory tbf = new ToolBarFactory(bundle, this);
	tb = tbf.createJToolBar("ToolBar");
	tb.setFloatable(true);

	// Create a math toolbar
	ToolBarFactory mathbar=new ToolBarFactory(bundle,this);
	mb=mathbar.createJToolBar("MathBar");
	mb.setFloatable(true);

        //Create a draw toolbar
        //ToolBarFactory drawbar=new ToolBarFactory(bundle,this);
	//drawb=drawbar.createJToolBar("DrawBar");
        //drawb.setFloatable(true);

	//Add these two components to the north panel.
	northpane.add("North",jmb);
	northpane.add("Center",tb);
	//northpane.add("West",drawb);
	northpane.add("South",mb);
	
        // Create the status bar
	statusBar=new StatusBar();
	
        //Create the language dialog
        languageDialog = new LanguageDialog(main);
        languageDialog.setLanguageChangeHandler(this);
	
        // Create the user style dialog
        userStyleDialog = new UserStyleDialog(main);
        userStyleDialog.setChangeHandler(this);
	
	//Add the north pane, canvas, and status bar.
	JLayeredPane canvaspane=new JLayeredPane();
	canvaspane.setOpaque(false);
	//I make the graphic image transparent and put it
	//on top.  Hence the order below.
	canvaspane.add(thecanvas,new Integer(10));
//    	canvaspane.add(textcanvas,new Integer(10));
//    	canvaspane.add(graphcanvas,new Integer(20));

	container.add("North",northpane);
	container.add("Center",canvaspane);
	container.add("South",statusBar);
	
	//Create the text field object
	//Font f=main.getFont(); //new Font("SansSerif",Font.PLAIN,18);
        /*	textfield=new JTextField(25);
	textfield.setFont(f);
	textfield.addActionListener(this);
	textfield.setOpaque(false);
	textfield.setBorder(new EmptyBorder(textfield.getInsets()));
	textfield.setBounds(100,100,textfield.getPreferredSize().width,
			    textfield.getPreferredSize().height);

	textpane=new JPanel();
	textpane.setLayout(null);
	textpane.setOpaque(false);
	textpane.setBounds(0,80,resources.getInteger("Frame.width"),
			    resources.getInteger("Frame.height")-60);
	textpane.addMouseListener(new MouseAdapter() {
		public void mouseClicked(MouseEvent e) {
		    textfield.requestFocus();
  		    textfield.setLocation(e.getX(),e.getY());
		    textfield.setVisible(true);
		}
	    });

        textpane.addMouseMotionListener(new MouseMotionAdapter() {
                public void mouseMoved(MouseEvent e) {
                    statusBar.setXPosition(e.getX());
                    statusBar.setYPosition(e.getY());
                }
            });

	textpane.add(textfield);
        */

	mainContain=main.getContentPane();
	layercont=new JLayeredPane();
	layercont.add(container,new Integer(1));
	//layercont.add(textpane,new Integer(4));
	mainContain.add(layercont);

	//Add event listeners to the canvas.
//          graphcanvas.setZoomHandler(this);
//  	textcanvas.setZoomHandler(this);
	thecanvas.setZoomHandler(this);
    }
    
    /**
     * This method correctly creates a new, blank SVG document.
     */
    public SVGOMDocument initializeDoc(SVGOMDocument doc) {
	SVGGraphics2D svgbase=new SVGGraphics2D(new SVGOMDocument());
	try{
    	    CharArrayWriter out=new CharArrayWriter();
    	    BufferedWriter bufOut=new BufferedWriter(out);
    	    svgbase.stream(bufOut,true);
    	    char[] charArray=out.toCharArray();
    	    CharArrayReader charReader=new CharArrayReader(charArray);
    	    BufferedReader bufRead=new BufferedReader(charReader);
      	    doc=df.createDocument("file:/tmp",new InputSource(bufRead));
    	    out.flush();
    	    out.close();
    	    bufOut.flush();
    	    bufOut.close();
	}
	
	catch(Exception ex) {
	    ex.printStackTrace();
	}
	
	return doc;
    }
	
    /**
     * Handles <enter> events from the JTextField object.
     */
    public void actionPerformed(ActionEvent e) {
	paintText(textfield.getX(),textfield.getY());
    }

    /**
     * Tells the viewer whether or not it must be set to the size
     * of the loaded documents.
     */
    public void setFixedSize(boolean b) {
        fixedSize = b;
        // !!! TODO
    }

    /**
     * Returns the fixedSize field value.
     */
    public boolean isFixedSize() {
        return fixedSize;
    }

    /**
     * Returns the current loading thread if one.
     */
    public Thread getLoadingThread() {
        return thread;
    }

    /**
     * Called when the language settings change.
     */
    public void languageChanged(String lang) {
        userLanguages = lang;
    }

    /**
     * Called when the user stylesheet has changed.
     */
    public void userStyleSheetURIChanged(String s) {
        userStyleSheetURI = s;
    }

    public void zoomChanged(float f) {
        statusBar.setZoom(f);
    }

    // UserAgent ///////////////////////////////////////////////////

    /**
     * Returns the default size of the viewport of this user agent.
     */
    public Dimension2D getViewportSize() {
	//        return textcanvas.getSize();
        return thecanvas.getSize();
    }

    /**
     * Returns the <code>EventDispatcher</code> used by the
     * <code>UserAgent</code> to dispatch events on GVT.
     */
    public EventDispatcher getEventDispatcher() {
        return eventDispatcher;
    }

    /**
     * Displays an error message in the User Agent interface.
     */
    public void displayError(final String msg) {
        Runnable r = new Runnable() {
            public void run() {
                JOptionPane pane =
                    new JOptionPane(msg, JOptionPane.ERROR_MESSAGE);
                JDialog dialog = pane.createDialog(ViewerFrame.this, "ERROR");
                dialog.show();
            }
        };
        if (!EventQueue.isDispatchThread()) {
            EventQueue.invokeLater(r);
        } else {
            r.run();
        }
    }

    /**
     * Displays an error resulting from the specified Exception.
     */
    public void displayError(final Exception ex) {
        displayError(ex.getMessage());
    }

    /**
     * Displays a message in the User Agent interface.
     */
    public void displayMessage(String message) {
        statusBar.setMainMessage(message);
    }

    /**
     * Returns the pixel to mm factor.
     */
    public float getPixelToMM() {
        return 0.33f;
    }

    /**
     * Returns the language settings.
     */
    public String getLanguages() {
        return userLanguages;
    }

    /**
     * Returns the user stylesheet uri.
     * @return null if no user style sheet was specified.
     */
    public String getUserStyleSheetURI() {
        return userStyleSheetURI;
    }

    /**
     * Opens a link.
     * @param elt The activated link element.
     */
    public void openLink(SVGAElement elt) {
        main.openLink(this, elt);
    }

    /**
     * Informs the user agent to change the cursor.
     * @param cursor the new cursor
     */
    public void setSVGCursor(Cursor cursor) {
        // XXX: immediate set,  but may not update "requested" value
	//        canvas.setCursor(cursor);
	textpane.setCursor(cursor);
    }

    /**
     * Runs the given thread.
     */
    public synchronized void runThread(Thread t) {
        stopAction.update(true);
        reloadAction.update(false);
        thread = t;
        thread.start();
    }

    /**
     * Returns the class name of the XML parser.
     */
    public String getXMLParserClassName() {
        return main.getXMLParserClassName();
    }

    /**
     * Returns the <code>AffineTransform</code> currently
     * applied to the drawing by the UserAgent.
     */
    public AffineTransform getTransform() {
        //return graphcanvas.getTransform();
	return thecanvas.getTransform();
    }

    /**
     * Returns the location on the screen of the
     * client area in the UserAgent.
     */
    public Point getClientAreaLocationOnScreen() {
        //return graphcanvas.getLocationOnScreen();
	return thecanvas.getLocationOnScreen();
    }

    /**
     * Loads the given document.
     * @param s The document name.
     */
    public void loadDocument(String s) {
        uri = s;
        File f = new File(uri);
        if (f.exists()) {
            if (f.isDirectory()) {
                uri = null;
            } else {
                uri = "file:" + uri;
            }
        }
        if (uri != null) {
            // interrupt any document load already underway
            if ((thread != null) && thread.isAlive()) {
                thread.interrupt();
            }
            loadedDocument++;
            if (loadedDocument == loadedDocuments.size()) {
                loadedDocuments.add(uri);
            } else {
                Object o = loadedDocuments.get(loadedDocument);
                if (!o.equals(uri)) {
                    loadedDocumentsCount = loadedDocument + 1;
                }
                loadedDocuments.set(loadedDocument, uri);
            }
            if (loadedDocumentsCount == loadedDocument) {
                loadedDocumentsCount++;
            }
            backAction.update();
            forwardAction.update();

//              locationBar.setText(uri);
            Thread t = DocumentLoadRunnable.createLoaderThread(uri,this,df);
            runThread(t);
        }
    }

    /**
     * Append the fragment to the main svg document.
     */
    public void appendFragToDoc(JSVGCanvas thecanvas,SVGOMDocument thedoc) {
	//writeOutDoc(thedoc);
	setCursor(WAIT_CURSOR);
	setSVGCursor(WAIT_CURSOR);
	thecanvas.setSVGDocument(null);
	reloadAction.update(false);
	stopAction.update(true);
	statusBar.setMainMessage(resources.getString("Document.loading"));
	
	//New doc has been loaded, prepare for new view
	//DefaultSVGContext dc = new DefaultSVGContext();
	dc=new DefaultSVGContext();
  	dc.setUserAgent(this);
	dc.setUserStyleSheetURI(userStyleSheetURI);

    	thedoc.setSVGContext(dc);
    	domViewer.setDocument(thedoc,
    			      (ViewCSS)thedoc.getDocumentElement());
    	statusBar.setMainMessage(resources.getString("Document.creating"));
    	thecanvas.setSVGDocument(thedoc);
	
	setCursor(DEFAULT_CURSOR);
	setSVGCursor(DEFAULT_CURSOR);
	stopAction.update(false);
	reloadAction.update(true);
	statusBar.setMainMessage("");
	statusBar.setMessage(resources.getString("Document.done"));
    }

    /**
     * This method loads an SVGOMDocument that was created internally
     * by another component to the specified canvas.
     */
    public void loadInternalDocument(JSVGCanvas thecanvas,
				     SVGOMDocument thedoc) {

	setCursor(WAIT_CURSOR);
	setSVGCursor(WAIT_CURSOR);
	thecanvas.setSVGDocument(null);
	reloadAction.update(false);
	stopAction.update(true);
	statusBar.setMainMessage(resources.getString("Document.loading"));
	
	//New doc has been loaded, prepare for new view
	//DefaultSVGContext dc = new DefaultSVGContext();
	dc = new DefaultSVGContext();
	
	dc.setUserAgent(ViewerFrame.this);
	dc.setUserStyleSheetURI(userStyleSheetURI);
	thedoc.setSVGContext(dc);
	domViewer.setDocument(thedoc,
			      (ViewCSS)thedoc.getDocumentElement());
	statusBar.setMainMessage(resources.getString("Document.creating"));
	
	thecanvas.setSVGDocument(thedoc);
	setCursor(DEFAULT_CURSOR);
	stopAction.update(false);
	reloadAction.update(true);
	statusBar.setMainMessage("");
	statusBar.setMessage(resources.getString("Document.done"));

    }
    
    // ActionMap /////////////////////////////////////////////////////

    /**
     * The map that contains the action listeners
     */
    protected Map listeners = new HashMap();

    /**
     * Returns the action associated with the given string
     * or null on error
     * @param key the key mapped with the action to get
     * @throws MissingListenerException if the action is not found
     */
    public Action getAction(String key) throws MissingListenerException {
        Action result = (Action)listeners.get(key);
        if (result == null) {
	    //            result = textcanvas.getAction(key);
            result = thecanvas.getAction(key);
        }
        return result;
    }

    /**
     * This method is called whenever you want to convert the text
     * the user has typed into SVG.
     */
    public void paintText(int xpos,int ypos) {
	textfield.selectAll();
	wbtext.test();
	wbtext.loadDataFile(textfield.getText());
	textfield.setVisible(false);
	textfield.cut();
	wbtext.drawShape(main.getSVGGenerator(),xpos,ypos);
    }

    /**
     * Closes the frame.
     */
    public void close() {
        if (main.closeFrame(this)) {
            setVisible(false);
        }
    }


    // Actions //////////////////////////////////////////////////////////

    /**
     * Take action on receipt of a document event.
     */
    public void processDocumentEvent(DocumentEvent e) {

        // perhaps keying on public field is faster than instanceof ?

        if (e.classid == DocumentEvent.LOADING) {
            processDocumentLoadingEvent((DocumentLoadingEvent)e);
        } else if (e.classid == DocumentEvent.PROPERTY) {
            processDocumentPropertyEvent((DocumentPropertyEvent)e);
        }
    }

    /**
     * Take action on receipt of a document loading event.
     */
    public void processDocumentLoadingEvent(DocumentLoadingEvent e) {
        switch (e.type) {
        case (DocumentLoadingEvent.START_LOADING):
            setCursor(WAIT_CURSOR);
            setSVGCursor(WAIT_CURSOR);
            //graphcanvas.setSVGDocument(null);
	    thecanvas.setSVGDocument(null);
            reloadAction.update(false);
            stopAction.update(true);
            statusBar.setMainMessage(resources.getString("Document.loading"));
            break;
        case (DocumentLoadingEvent.LOADED):
            // New doc has been loaded, prepare for new view
            DefaultSVGContext dc = new DefaultSVGContext();
            graphdoc = (SVGOMDocument) e.getValue();
	    //	    main.setSVGOMDocument(svgomdoc);
            dc.setUserAgent(ViewerFrame.this);
            dc.setUserStyleSheetURI(userStyleSheetURI);
            graphdoc.setSVGContext(dc);
            domViewer.setDocument(graphdoc,(ViewCSS)graphdoc.getDocumentElement());
            statusBar.setMainMessage(resources.getString("Document.creating"));
            break;
        case (DocumentLoadingEvent.DONE):
            graphdoc = (SVGOMDocument) e.getValue();
            //graphcanvas.setSVGDocument(graphdoc);
	    thecanvas.setSVGDocument(graphdoc);
            setCursor(DEFAULT_CURSOR);
            stopAction.update(false);
            reloadAction.update(true);
            statusBar.setMainMessage("");
            statusBar.setMessage(resources.getString("Document.done"));
            break;
        case (DocumentLoadingEvent.LOAD_CANCELLED):
            setCursor(DEFAULT_CURSOR);
            stopAction.update(false);
            reloadAction.update(true); // in case we want to try again
            statusBar.setMainMessage("");
            statusBar.setMessage(resources.getString("Document.cancelled"));
            break;
        case (DocumentLoadingEvent.LOAD_FAILED):
            setCursor(DEFAULT_CURSOR);
            stopAction.update(false);
            reloadAction.update(true); // in case we want to try again
            statusBar.setMainMessage("");
            statusBar.setMessage(resources.getString("Document.failed"));
            displayError(e.getException());
        }
    }

    public void writeOutDoc(SVGOMDocument testdoc) {

	try {
  	    Writer stdout=new OutputStreamWriter(System.out,"UTF-8");
  	    System.out.println("Updated document");
	    XmlWriter.writeXml(testdoc,stdout);
	    stdout.flush();
	}
	catch(Exception ex) {ex.printStackTrace();}

    }

    /**
     * Take action on receipt of a document property change.
     */
    //Comment for now while converting ViewerFrame to JApplet.
    public void processDocumentPropertyEvent(DocumentPropertyEvent e) {
//  	switch (e.type) {
//  	case (DocumentPropertyEvent.TITLE) :
//  	    String title = (String) e.getValue();
//            if (title.equals("")) {
//                setTitle(resources.getString("Frame.title") + ": " +
//                         resources.getString("Frame.no_title"));
//            } else {
//                setTitle(resources.getString("Frame.title") + ": " + title);
//            }
//            break;
//  	case (DocumentPropertyEvent.SIZE) :
//  	    Dimension size = (Dimension) e.getValue();
//  	    if (!canvas.getSize().equals(size)) {
//  		canvas.setPreferredSize(size);
//  		panel.invalidate();
//  		if (!fixedSize) {
//  		    pack();
//  		}
//  	    }
//  	    break;
//  	case (DocumentPropertyEvent.DESCRIPTION) :
//  	    String desc = (String) e.getValue();
//  	    if (desc.equals("")) {
//  		desc = resources.getString("Description.no_description");
//  	    }
//  	    description = desc;
//  	    break;
//  	}
    }

    
    //============================================================

    /**
     * Define below the accessor methods.
     */   

    public String getDescription() { return description; }
    public String getUri() { return uri; }
    public ResourceManager getResources() { return resources; }
    
    public int getLoadedDocument() { return loadedDocument; }
    public java.util.List getLoadedDocumentList() { return loadedDocuments; }
    public int getLoadedDocumentsCount() { return loadedDocumentsCount; }

//      public JSVGCanvas getGraphCanvas() { return graphcanvas; }
//      public JSVGCanvas getTextCanvas() { return textcanvas; }
    public JSVGCanvas getTheCanvas() { return thecanvas; }

    public String getCurrentExportPath() { return currentExportPath; }
    public StatusBar getStatusBar() { return statusBar; }

    public boolean getFixedSize() { return fixedSize; }
    
    public LanguageDialog getLanguageDialog() { return languageDialog; }
    public String getUserLanguages() { return userLanguages; }

    public LocationBar getLocationBar() { return locationBar; }

    public MemoryMonitor getMemoryMonitor() { 
	return (MemoryMonitor)memoryMonitor; 
    }

    public Main getMain() { return main; }
    public Application getApplication() { return main; }

    public URIChooser getURIChooser() { return uriChooser; }
    
    public boolean getProgressivePaintEnabled() {
	return progressivePaintEnabled; 
    }

    public SVGDocumentFactory getDocFactory() { return df; }
    
    public Thread getThread() { return thread; }

    public JFrame getThumbnailFrame() { return thumbnailFrame; }

    public DOMViewer getDOMViewer() { return domViewer; }
    
    public UserStyleDialog getUserStyleDialog() { return userStyleDialog; }

    public char[] getBuffer() { return buffer; }

    public String getCurrentPath() { return currentPath; }
    
    public ResourceBundle getBundle() { return bundle; }
	
    public JPanel getContainerFrame() { return container; }
    
    public JPanel getpanel() { return panel; }
    
    public JButton getTextButton() { return textButton; }

    public SVGOMDocument getTextDoc() { return textdoc;} 
    public SVGOMDocument getGraphDoc() { return graphdoc; }
    public SVGOMDocument getTheDoc() { return thedoc; }

    public void setTextDocument(SVGOMDocument doc) { textdoc=doc; }
    public void setGraphDocument(SVGOMDocument doc) { graphdoc=doc; }
    public void setTheDocument(SVGOMDocument doc) { thedoc=doc; }

    //============================================================

    /**============================================================
     * The following inner classes define actions for the buttons.
     *============================================================*/
    
    /**
     * Define the open page action.
     */
    public class OpenPageAction extends AbstractAction {
	
	public OpenPageAction() {}
	
	public void actionPerformed(ActionEvent e) {
	    uriChooser.pack();
	    
	    Rectangle fr = getBounds();
	    Dimension ud = uriChooser.getSize();
	    uriChooser.setLocation(fr.x + (fr.width  - ud.width) / 2,
				   fr.y + (fr.height - ud.height) / 2);
	    
	    uriChooser.show();
	}
    }
    
    /**
     * The URIChooserOKAction definition.
     */
    public class URIChooserOKAction extends AbstractAction {
	public URIChooserOKAction() {}
	public void actionPerformed(ActionEvent e) {
	    loadDocument(uriChooser.getText());
	}
    }

    /**
     * The action to place the integral sign.
     */
    public class IntegralAction extends AbstractAction {
	public IntegralAction() {}
	public void actionPerformed(ActionEvent e) {

          if(layercont.getIndexOf(textpane) != -1) {
	    String text=textfield.getText();
	    text+=" <int> ";
	    textfield.setText(text);
          } else {
            System.out.println("You are not in text mode, Please click the text button first.");
          }
	}
    }

    /**
     * The action to place the square root sign.
     */
    public class SqrtAction extends AbstractAction {
	public SqrtAction() {}
	public void actionPerformed(ActionEvent e) {
          if(layercont.getIndexOf(textpane) != -1) {
	    String text=textfield.getText();
	    text+=" <sqr> ";
	    textfield.setText(text);
          } else {
            System.out.println("You are not in text mode, Please click the text button first.");
          }
	}
    }

    /**
     * The action to place the partial symbol.
     */
    public class PartAction extends AbstractAction {
	public PartAction() {}
	public void actionPerformed(ActionEvent e) {
          if(layercont.getIndexOf(textpane) != -1) {
	    String text=textfield.getText();
	    text+=" <par> ";
	    textfield.setText(text);
          } else {
            System.out.println("You are not in text mode, Please click the text button first.");
          }
	}
    }

    /**
     * The action to place the delta symbol.
     */
    public class DelAction extends AbstractAction {
	public DelAction() {}
	public void actionPerformed(ActionEvent e) {
          if(layercont.getIndexOf(textpane) != -1) {
	    String text=textfield.getText();
	    text+=" <del> ";
	    textfield.setText(text);
          } else {
            System.out.println("You are not in text mode, Please click the text button first.");
          }
	}
    }

  /**
   * The action to place the pencil off and save SVG.
   */
  public class PenOffAction extends AbstractAction {
    public PenOffAction() {}
    public void actionPerformed(ActionEvent e) {
      System.out.println("PenOffAction: ");
      if(layercont.getIndexOf(drawpanel) != -1) {
        wbdraw.test();
        wbdraw.drawShape(sg2d,0,0);      
        Graphics g = drawpanel.getGraphics();
        drawpanel.clearpaint(g);
      } else {
        System.out.println("You are not in Pen-On mode, Please click the Pen-On button first.");
      }
    }
  }

  /**
   * The action to draw some image.
   */
  public class PenOnAction extends AbstractAction {
    //DrawPanel drawpanel;

    public PenOnAction() {
      drawpanel = new DrawPanel();
      drawpanel.setLayout(null);
      drawpanel.setOpaque(false);
      drawpanel.setBounds(0,80,resources.getInteger("Frame.width"),
			    resources.getInteger("Frame.height")-60);
    }    

    public void actionPerformed(ActionEvent e) {
      System.out.println("PencilAction: "+e.getActionCommand());
      System.out.println("Layer Pencil: "+layercont.getLayer(textpane));
      System.out.println("LayerIndex Pencil: "+layercont.getIndexOf(textpane));

      if(layercont.getIndexOf(textpane) != -1) {
        layercont.remove(layercont.getIndexOf(textpane));
      }
      if(layercont.getIndexOf(drawpanel) == -1) {
        layercont.add(drawpanel,new Integer(5));
      }
    }
  }
  /**
   * The action to place the Text Field.
   */
  public class TextAction extends AbstractAction {
    public TextAction() {

      Font f=main.getFont(); //new Font("SansSerif",Font.PLAIN,18);
      textfield=new JTextField(25);
      textfield.setFont(f);
      //textfield.addActionListener(this);
      textfield.setOpaque(false);
      textfield.setBorder(new EmptyBorder(textfield.getInsets()));
      textfield.setBounds(100,100,textfield.getPreferredSize().width,
			    textfield.getPreferredSize().height);

      textfield.addActionListener( new ActionListener() {
    	public void actionPerformed(ActionEvent evt) {
          System.out.println("Text pane: "+evt.getActionCommand());
          paintText(textfield.getX(),textfield.getY());
    	}	   	
        });

      textpane=new JPanel();
      textpane.setLayout(null);
      textpane.setOpaque(false);
      textpane.setBounds(0,80,resources.getInteger("Frame.width"),
			    resources.getInteger("Frame.height")-60);
      textpane.addMouseListener(new MouseAdapter() {
          public void mouseClicked(MouseEvent e) {
            textfield.requestFocus();
            textfield.setLocation(e.getX(),e.getY());
            textfield.setVisible(true);
          }
        });

      textpane.addMouseMotionListener(new MouseMotionAdapter() {
          public void mouseMoved(MouseEvent e) {
            statusBar.setXPosition(e.getX());
            statusBar.setYPosition(e.getY());
          }
        });

      textpane.add(textfield);
    }

    public void actionPerformed(ActionEvent e) {

      System.out.println("TextAction: "+e.getActionCommand());
      System.out.println("Layer: "+layercont.getLayer(textpane));
      System.out.println("LayerIndex: "+layercont.getIndexOf(textpane));
      if(layercont.getIndexOf(drawpanel) != -1) {
        layercont.remove(layercont.getIndexOf(drawpanel));
      }

      if(layercont.getIndexOf(textpane) == -1) {
        layercont.add(textpane,new Integer(4));
        //mainContain.add(layercont);
      }
    }
  }


  /* Draw image, just scribble feature added
   */
  public class DrawPanel extends JPanel implements MouseListener, MouseMotionListener {
    Vector lines = new Vector();
    Vector colors = new Vector();
    int x1,y1;
    int x2,y2;
    int xl, yl;

    public DrawPanel() {
      setBackground(Color.white);
      setForeground(Color.black);
      addMouseMotionListener(this);
      addMouseListener(this);
    }
    
    public void mouseDragged(MouseEvent e) {
      e.consume();
      colors.addElement(getForeground());
      lines.addElement(new Rectangle(x1, y1, e.getX(), e.getY()));
      x1 = e.getX();
      y1 = e.getY();

      repaint();
    }

    public void mouseMoved(MouseEvent e) {
      statusBar.setXPosition(e.getX());
      statusBar.setYPosition(e.getY());
    }

    public void mousePressed(MouseEvent e) {
      e.consume();
      colors.addElement(getForeground());
      lines.addElement(new Rectangle(e.getX(), e.getY(), -1, -1));
      x1 = e.getX();
      y1 = e.getY();
      repaint();
    }

    public void mouseReleased(MouseEvent e) {
      e.consume();
    }
    
    public void mouseEntered(MouseEvent e) {
    }

    public void mouseExited(MouseEvent e) {
    }

    public void mouseClicked(MouseEvent e) {
    }
    
    public void paintComponent(Graphics g){
      super.paintComponent(g);

      Graphics2D g2d = (Graphics2D) g;
      int np = lines.size();
      
      /* draw the current lines */
      sg2d.setColor(getForeground());
      sg2d.setPaintMode();
      g2d.setColor(getForeground());
      g2d.setPaintMode();
      for (int i=0; i < np; i++) {
        Rectangle p = (Rectangle)lines.elementAt(i);
        sg2d.setColor((Color)colors.elementAt(i));
        g2d.setColor((Color)colors.elementAt(i));
        if (p.width != -1) {
          sg2d.drawLine(p.x, p.y, p.width, p.height);
          g2d.drawLine(p.x, p.y, p.width, p.height);
        } else {
          sg2d.drawLine(p.x, p.y, p.x, p.y);
          g2d.drawLine(p.x, p.y, p.x, p.y);
        }
      }
    }

    public void clearpaint(Graphics g) {
      //g.setColor(getForeground());
      lines = new Vector();
      colors = new Vector();
      paintComponent(g);
    }

  }
  
  public class ProgressivePaintAction extends AbstractAction {
	public ProgressivePaintAction() {}

	public void actionPerformed(ActionEvent e) {
	    progressivePaintEnabled=
		((JCheckBoxMenuItem)e.getSource()).isSelected();
	    //textcanvas.setProgressiveRenderingEnabled(progressivePaintEnabled);
	    thecanvas.setProgressiveRenderingEnabled(progressivePaintEnabled);
	}
    }
    
    /**
     * Opens an existing SVG document from disk.
     */
    public class OpenAction extends AbstractAction {
	public OpenAction() {}

	public void actionPerformed(ActionEvent e) {
          System.out.println("OpenAction: "+e.getActionCommand());

	    JFileChooser fileChooser =
		new JFileChooser(currentPath);
	    fileChooser.setFileHidingEnabled(false);
	    fileChooser.setFileSelectionMode
		(JFileChooser.FILES_AND_DIRECTORIES);
	    fileChooser.setFileFilter(new SVGFileFilter());
	    
	    int choice = fileChooser.showOpenDialog(ViewerFrame.this);
	    if (choice == JFileChooser.APPROVE_OPTION) {
		File f = fileChooser.getSelectedFile();
		try {
		    loadDocument(currentPath = f.getCanonicalPath());
		} 
		catch (IOException ex) {
		}
	    }
	}
    }

    /**
     * Loads a data file and displays it.
     */
    public class LoadDataAction extends AbstractAction {
	public LoadDataAction() {}

	public void actionPerformed(ActionEvent e) {
	    GraphicsManager graphman;
	    String[] placebo=new String[1];
	    JFileChooser fileChooser =
		new JFileChooser(currentPath);
	    fileChooser.setFileHidingEnabled(false);
	    fileChooser.setFileSelectionMode
		(JFileChooser.FILES_AND_DIRECTORIES);
	    fileChooser.setFileFilter(new XSILFileFilter());
	    
	    int choice = fileChooser.showOpenDialog(ViewerFrame.this);
	    if (choice == JFileChooser.APPROVE_OPTION) {
		File f = fileChooser.getSelectedFile();
		try {
		    System.out.println("Load the data file and grok");
		    graphman=main.getGraphicsManager();
		    graphman.test();
  		    graphman.initChildren();
		    placebo[0]=f.getCanonicalPath();
		    graphman.loadDataFiles(placebo);
 		    graphman.drawChildren(main.getSVGGenerator());
		} 
		catch (Exception ex) {
		}
	    }
	}
    }

    /**
     * Saves the current SVG image to a file. Currenly only 
     * writes out the text document.  Later, will need to also
     * write out the graphics/image document.
     */
    public class SaveSVGAction extends AbstractAction {
	public SaveSVGAction() {}

	public void actionPerformed(ActionEvent e) {
	    JFileChooser fileChooser =
		new JFileChooser(currentPath);
	    fileChooser.setFileHidingEnabled(false);
	    fileChooser.setFileSelectionMode
		(JFileChooser.FILES_AND_DIRECTORIES);
	    fileChooser.setFileFilter(new SVGFileFilter());
	    
	    int choice = fileChooser.showSaveDialog(ViewerFrame.this);
	    if (choice == JFileChooser.APPROVE_OPTION) {
		File f = fileChooser.getSelectedFile();

  		try {
		    Writer writeout=new PrintWriter(new FileWriter(f));
  		    XmlWriter.writeXml(thedoc,writeout); //Change from textdoc to thedoc
		    writeout.flush();
  		} 
  		catch (Exception ex) {
  		}
	    }
	}
    }

    /**
     * Exports the image as JPG.  Currenly only writes out the 
     * text document.
     */
    public class ExportJPGAction extends AbstractAction {
	public ExportJPGAction(ViewerFrame vf){
	}
	public void actionPerformed(ActionEvent e) {
	    JFileChooser fileChooser =
		new JFileChooser(currentExportPath);
            fileChooser.setFileHidingEnabled(false);
            fileChooser.setFileSelectionMode
                (JFileChooser.FILES_AND_DIRECTORIES);
	    
            int choice = fileChooser.showSaveDialog(ViewerFrame.this);
            if (choice == JFileChooser.APPROVE_OPTION) {
                final File f = fileChooser.getSelectedFile();
                //BufferedImage buffer = textcanvas.getBuffer();
		BufferedImage buffer = thecanvas.getBuffer();
                // create a BufferedImage of the appropriate type
                int w = buffer.getWidth();
                int h = buffer.getHeight();
                final ImageTranscoder trans = new JpegTranscoder();
                final BufferedImage img = trans.createImage(w, h);
                // paint the buffer to the image
                Graphics2D g2d = img.createGraphics();
                g2d.drawImage(buffer, null, 0, 0);
                new Thread() {
			public void run() {
			    try {
				currentExportPath = f.getCanonicalPath();
				OutputStream ostream =
				    new BufferedOutputStream(new FileOutputStream(f));
				trans.writeImage(img, ostream);
				ostream.flush();
				ostream.close();
				statusBar.setMessage(
						     resources.getString("Document.export"));
			    } catch (IOException ex) { }
			}
		    }.start();
            }
	}
    }
    
    /**
     * Exports the image as PNG.
     */
        public class ExportPNGAction extends AbstractAction {
        public void actionPerformed(ActionEvent e) {
            JFileChooser fileChooser =
                new JFileChooser(currentExportPath);
            fileChooser.setFileHidingEnabled(false);
            fileChooser.setFileSelectionMode
                (JFileChooser.FILES_AND_DIRECTORIES);

            int choice = fileChooser.showSaveDialog(ViewerFrame.this);
            if (choice == JFileChooser.APPROVE_OPTION) {
                final File f = fileChooser.getSelectedFile();
                //BufferedImage buffer = graphcanvas.getBuffer();
		BufferedImage buffer = thecanvas.getBuffer();
                // create a BufferedImage of the appropriate type
                int w = buffer.getWidth();
                int h = buffer.getHeight();
                final ImageTranscoder trans = new PngTranscoder();
                final BufferedImage img = trans.createImage(w, h);
                // paint the buffer to the image
                Graphics2D g2d = img.createGraphics();
                g2d.drawImage(buffer, null, 0, 0);
                new Thread() {
                    public void run() {
                        try {
                            currentExportPath = f.getCanonicalPath();
                            OutputStream ostream =
                              new BufferedOutputStream(new FileOutputStream(f));
                            trans.writeImage(img, ostream);
                            ostream.flush();
                            ostream.close();
                            statusBar.setMessage(
                                resources.getString("Document.export"));
                        } catch (IOException ex) { }
                    }
                }.start();
            }
        }
    }


    /**
     * Displays the "about" dialog.
     */
    public class AboutAction 
	extends AbstractAction implements DocumentListener {
	JSVGCanvas canvas;
	JFrame aboutFrame;
	public AboutAction() {}
	public void actionPerformed(ActionEvent e) {
	    if (aboutFrame == null) {
		aboutFrame = new JFrame(resources.getString("About.title"));
		aboutFrame.setSize(resources.getInteger("About.width"),
				   resources.getInteger("About.height"));
		JPanel panel = new JPanel(new BorderLayout());
		
		aboutFrame.getContentPane().add(panel);
		panel.setBorder(BorderFactory.createCompoundBorder
				(BorderFactory.createTitledBorder
				 (BorderFactory.createEtchedBorder(),
				  resources.getString("About.border_title"),
				  TitledBorder.CENTER,
				  TitledBorder.DEFAULT_POSITION),
				 BorderFactory.createEmptyBorder(5, 5, 5, 5)));
		
		JPanel p = new JPanel(new FlowLayout(FlowLayout.RIGHT));
		ButtonFactory bf = new ButtonFactory(bundle, null);
		JButton button;
		p.add(button = bf.createJButton("AboutCloseButton"));
		button.addActionListener(new AbstractAction() {
			public void actionPerformed(ActionEvent e) {
			    aboutFrame.setVisible(false);
			}
		    });
		aboutFrame.getContentPane().add("South", p);
		canvas = new JSVGCanvas(ViewerFrame.this);
		canvas.setBorder(BorderFactory.createLoweredBevelBorder());
		panel.add(canvas);
		
		String uri =
		    getClass().getResource("resources/authors.svg").toString();
		Thread t = DocumentLoadRunnable.createLoaderThread(uri,
								   this,
								   df);
		
		t.start();
	    }
	    Rectangle fr = getBounds();
	    Dimension sd = aboutFrame.getSize();
	    aboutFrame.setLocation(fr.x + (fr.width  - sd.width) / 2,
				   fr.y + (fr.height - sd.height) / 2);
	    aboutFrame.show();
	}

	/**
	 * Take action on receipt of a DocumentEvent.
	 */
	public void processDocumentEvent(DocumentEvent e) {
	    if (e.classid == DocumentEvent.LOADING) {
		if (e.type == DocumentLoadingEvent.DONE) {
		    DefaultSVGContext dc = new DefaultSVGContext();
		    SVGOMDocument doc = (SVGOMDocument)e.getValue();
		    dc.setUserAgent(ViewerFrame.this);
		    doc.setSVGContext(dc);
		    canvas.setSVGDocument(doc);
		}
	    } else if (e.classid == DocumentEvent.PROPERTY) {
		//
	    }
	}   
    }
    
    /**
     * Make thumbnail.
     */ 
    public class ThumbnailAction extends AbstractAction {
	ViewerFrame vf;
	ResourceManager resources;
	JFrame thumbnailFrame;
	JSVGCanvas canvas;
	
	public ThumbnailAction() {}
	
	public ThumbnailAction(ViewerFrame vf){
	    this.vf=vf;
	    resources=vf.getResources();
	    thumbnailFrame=vf.getThumbnailFrame();
	    //canvas=vf.getGraphCanvas();
	    canvas=vf.getTheCanvas();
	}
	
	public void actionPerformed(ActionEvent e) {
	    if (thumbnailFrame == null) {
		thumbnailFrame =
		    new JFrame(resources.getString("Thumbnail.title"));
		thumbnailFrame.setSize
		    (resources.getInteger("Thumbnail.width"),
		     resources.getInteger("Thumbnail.height"));
		
		thumbnailFrame.getContentPane().add(canvas.getThumbnail());
		
		URL url =
		    getClass().getResource(resources.getString("Frame.icon"));
		thumbnailFrame.setIconImage(new ImageIcon(url).getImage());
	    }
	    Rectangle fr = vf.getBounds();
	    Dimension td = thumbnailFrame.getSize();
	    thumbnailFrame.setLocation(fr.x + (fr.width  - td.width) / 2,
				       fr.y + (fr.height - td.height) / 2);
	    thumbnailFrame.show();
	}
    }
}
