import java.util.Properties;
import java.io.*;
import org.gxos.*;
import org.gxos.schema.*;
import org.gxos.schema.types.*;
import org.gxos.gndi.context.*;
import org.gxos.debug.Log;

/**
 * Example for object construction.
 * @author Ozgur Balsoy
 * @version 1.0
 */

 public class Example1 {

  public Example1(Properties env) {

	try {

            // With the given config values (either from the environment,
            // or configuration file) construct the initial context.
            // See main() first.
            //
            // Contexts in JNDI are named containers which can contain
            // other named contexts and objects. For each start, we need
            // a startup container which is the initial context. This is
            // constructed in ContextManager. Using getRootContext method
            // users of this API set their working environment.

            System.out.println("Initializing environment...");
            ContextManager.initialize(env);
            GxContext rootctx = ContextManager.getContext();

            // Probably you would want to group your objects under a name.
            // Collections serve to this purpose. Collections are
            // groups of objects, containers, or sub contexts for JNDI.
            // Under Garnet Collab. System, top level containers are called
            // mega meetings. Root context is also a container.
            // First create a container.

            GxContext objects = (GxContext)rootctx.createSubcontext("myroot");
            System.out.println("Created //myroot collection.");

            // Even, we can further put in them in sub contexts.
            // This time we will use different type of naming.
            // We can reach any of these subcontexts by using their names.
            // Hence we don't need to save them in a variable.

            rootctx.createSubcontext("myroot/users");
            System.out.println("Created //myroot/users collection.");
            // the same thing as the above line:
            // objects.createSubcontext("users");

            objects.createSubcontext("devices");
            System.out.println("Created //myroot/devices collection.");
            // the same thing as the above line:
            // rootctx.createSubcontext("myobjects/devices");

            // Now, let's create a user object, and place it under users.
            // The easy way (by factory functions):
            User u1 = (User)Users.createUser(
                new User(),     /* an empty user object. GXOS Schema doesn't support
                                   constructors with parameters. */
                "balsoy", "mypasswd", /* user name and a password */
                "Ozgur Balsoy", /* full name of the user first middle and last names.*/
                "", "",         /* prefix and suffixes if any, i.e. Dr., Jr., Ph.D. */
                "foo@bar.com"   /* email address */
                );

            // And, now let's get the user container.
            GxContext users = (GxContext) rootctx.lookup("myroot/users");

            // add the user to the container.
            users.bind("balsoy", u1);
            System.out.println("Added   //myroot/users/balsoy user.");

            // Second user
            User u2 = (User)Users.createUser(
                new User(),   /* an empty user object. GXOS Schema doesn't support
                                 constructors with parameters. */
                "jdoe", "foo", /* user name and a password */
                "John J Doe", /* full name of the user first middle and last names.*/
                "Dr", "",     /* prefix and suffixes if any, i.e. Dr., Jr., Ph.D. */
                "jdoe@foo.com" /* email address */
                );

            users.bind("jdoe", u2);
            System.out.println("Added   //myroot/users/jdoe user.");

            // Let's define a device
            Device d = (Device)Devices.createDevice(
                new Device(),           // an empty device object
                "localhost",   // device's name
                DeviceTypeType.UNIX     // others: Windows, Mac, PDA
            );

            // Add it to the device container.
            // This time, the full path is used.
            rootctx.bind("myroot/devices/localhost", d);
            System.out.println("Added   //myroot/device/localhost device.");
            System.out.println("\nTry Example2 to list these objects.");

            // We have the following structure now:
            //
            //    //                (root context)
            //    //myroot          (our root)
            //    //myroot/users    (users repository)
            //    //myroot/devices  (device repository)
            //    //myroot/users/balsoy     (gxos objects)
            //    //myroot/users/jdoe
            //    //myroot/devices/localhost

            // This example has constructed a sample object hierarchy.
            // Please see the next example for object retrievals

	} catch (Exception e) {
	    e.printStackTrace();
	}
    }



  public static void main(String[] args) {

    if(args.length > 0 && args[0].equals("-d"))
        Log.debug(true);

    try {
        String configFile = (String)System.getProperties().get(Defaults.GXOS_CONFIG);
        Properties env = Defaults.getConfiguration(configFile);

            //  alternative way of setting configuration
            /*
                env = new Properties();
                env.put(Context.INITIAL_CONTEXT_FACTORY,
                    "org.gxos.gndi.context.GxContextFactory");
                env.put(Context.PROVIDER_URL, "//");
                env.put(org.gxos.Defaults.GXOS_SUBJNDI_VAR,
                    "com.sun.jndi.fscontext.RefFSContextFactory");
                env.put(org.gxos.Defaults.GXOS_SUBJNDI_PROVIDER_VAR,
                    "file:///gxosroot");        // or wherever your GXOS root is.
            */

        Example1 example = new Example1(env);

    } catch(Exception e) {
        e.printStackTrace();
    }
  }
}